#!/usr/bin/perl -w

# LSB Test Execution Framework
# Web-Server Start Module (lsb-dtk-start.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Fcntl ':flock';
use File::Temp qw/tmpnam/;
use Cwd qw(abs_path);

my $port = shift;
if ($port) {
	if ($port !~ m/^\d+$/) {
		die "    Error: Invalid port '$port' specified!\n";
	}
}
else {
	$port = 8888;
}

my $sudo_cmd = shift;
if (!$sudo_cmd) {
	if (`lsb_release -ds 2>/dev/null` =~ m/ubuntu/i) {
		$sudo_cmd = 'sudo su -c';
	}
	else {
		$sudo_cmd = 'su -c';
	}
}

my $ask_pwd = ($< != 0);
my $dtk_dir = abs_path($0);
$dtk_dir =~ s!/[^/]+/[^/]+$!!;
my $app_data = '/var/opt/lsb/test/manager';

print "    The port '$port' will be used by the LSB DTK Manager web-UI server.\n";
if ($ask_pwd) {
	print "    The command '$sudo_cmd' will be used to gain root access.\n";
	print "    If you want to change this, run $0 <port> <sudo-command>\n\n";
}
else {
	print "    If you want to change this, run $0 <port>\n\n";
}

my $LOCK_FILE = "/tmp/dtk-server.lock";
if (-f $LOCK_FILE) {
	if (open(LOCKFILE, $LOCK_FILE)) {
		if (!flock(LOCKFILE, LOCK_EX | LOCK_NB)) {
			my $pid = <LOCKFILE>;
			$pid =~ s/^G:(\d+)$/$1/;
			die "    Error: An LSB DTK Manager instance is running on the port '$port' already! (PID: $pid.)\n";
		}
		else {
			flock(LOCKFILE, LOCK_UN);
		}
		close(LOCKFILE);
	}
}

if (system('ps -A | grep \'\\bdtk-server.pl\\b\' >/dev/null 2>&1') == 0) {
	die "    Error: An LSB DTK Manager instance is running already!\n";
}

$dtk_dir =~ s/([ ()\'])/\\$1/g;
my $cmd = "mkdir -p \"$app_data/results\" >/dev/null 2>&1; mkdir -p \"$app_data/log\" >/dev/null 2>&1; $dtk_dir/webui/dtk-server.pl $port &";
if ($ask_pwd) {
	$cmd = "$sudo_cmd '$cmd'";
}
my $res = system($cmd);
sleep(1);
if ($res != 0) {
	die "    Failed to start server.\n";
}
print "    Server started. Log file location:\n    $app_data/log/dtk-server.log.$port\n";

# Try to open start page in a browser
my $machine = `uname -n`;
$machine =~ s/[\r\n]//g;
print "\n    The start page should be opened in a browser shortly.\n    If it doesn't open, you can load it at http://$machine:$port\n";

my $browser_not_found = 5;

# First, try the Portland's xdg-open util.
if (system('which xdg-open >/dev/null 2>&1') == 0) {
	my $err_file = tmpnam();
	system("xdg-open \"http://127.0.0.1:$port\" >$err_file 2>&1");
	# An exit code of 0 indicates success.
	# 1 Error in command line syntax. 
	# 2 One of the files passed on the command line did not exist. 
	# 3 A required tool could not be found. 
	# 4 The action failed.
	$browser_not_found = $?;
	if ($browser_not_found) {
		system("cat $err_file");
	}
	unlink($err_file);
}
else {
	# Try to find a browser by it's name.
	foreach my $br_name qw(opera /usr/bin/mozilla-firefox firefox konqueror epiphany galeon mozilla) {
		if (system("which $br_name >/dev/null 2>&1") == 0) {
			system("$br_name \"http://127.0.0.1:$port\" >/dev/null 2>&1 &");
			$browser_not_found = 0;
			last;
		}
	}
}

if ($browser_not_found) {
	print "\n    Could not find a web-browser.\n    Go to 'http://$machine:$port' to configure and launch tests.\n";
}
