#!/usr/bin/perl -w

# LSB Test Execution Framework
# Web-Server Stop Module (lsb-dtk-stop.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Fcntl ':flock';
use Cwd qw(cwd abs_path);

sub confirm($) {
	print $_[0];
	$reply = <STDIN>;
	if (($reply !~ m/^y$/i) and ($reply !~ m/^yes$/i)) {
		exit(2);
	}
}

sub exec_sudo_cmd($$) {
	my ($cmd, $sudo_cmd) = @_;
	if ($sudo_cmd) {
		$cmd =~ s/\'/'\\\''/g;
		return system("$sudo_cmd '$cmd'");
	}
	else {
		return system($cmd);
	}
}

my $ask_pwd = ($< != 0);
my $dtk_dir = abs_path($0);
$dtk_dir =~ s!/[^/]+/[^/]+$!!;
my $whoami = `id -un`;
$whoami =~ s/[\r\n]//g;
my $sudo_cmd = '';

if ($ask_pwd) {
	$sudo_cmd = shift;
	if (!$sudo_cmd) {
		if (`lsb_release -ds 2>/dev/null` =~ m/ubuntu/i) {
			$sudo_cmd = 'sudo su -c';
		}
		else {
			$sudo_cmd = 'su -c';
		}
	}
	print "    The command '$sudo_cmd' will be used to gain root access.\n";
	print "    If you want to change this, run $0 <sudo-command>\n\n";
}

my $LOCK_FILE = '/tmp/dtk-server.lock';
my $pid;
if (open(LOCKFILE, $LOCK_FILE)) {
	if (!flock(LOCKFILE, LOCK_EX | LOCK_NB)) {
		$pid = <LOCKFILE>;
		$pid =~ s/^G:(\d+)$/$1/;
	}
	else {
		flock(LOCKFILE, LOCK_UN);
	}
	close(LOCKFILE);
}

if ($pid) {
	confirm('    Are you sure you want to stop the server? ');
	my $res = exec_sudo_cmd("cd $dtk_dir/utils; ./stop_server.pl $pid", $sudo_cmd);
	exit $res >> 8;
}
elsif (`ps -A 2>/dev/null` =~ m/dtk-server\.pl/) {
	confirm("    It seems that an old LSB DTK Manager is running.\n    killall will be used to stop it.\n    Continue? ");
	my $res = exec_sudo_cmd('killall dtk-server.pl', $sudo_cmd);
	exit $res >> 8;
}
else {
	die "    No running LSB DTK Manager was found.\n";
}
