# LSB Test Execution Framework
# FTP Routines Module (Ftp.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Ftp;
use strict;

use POSIX qw/mktime/;

my $ftp;
our $list_refresh = 0;
#----------------------------------------------------------------------

sub login_ftp {
	my ($srv) = @_;
	
	# Check FTP perl module
	if( system ("perl -MNet::FTP -e 1 >/dev/null 2>&1") != 0) {
			die "Perl module Net::FTP is required.";
	}
	
	# Open ftp
	require Net::FTP;
	
	print STDERR "FTP init... $srv\n";
	$ftp = Net::FTP->new($srv, Debug => 0, Passive => 1)
		or die "FTP init failed. Message: $@";
	
	# Log in
	print STDERR "FTP log in... $srv\n";
	$ftp->login()
		or die "FTP log in (anonymous) failed. Message: ".$ftp->message; 
	
	return;
}
#------------------------------------------------------------------------

# Reads FTP listing from a file.
sub read_listing_from_file {
	my ($filename) = @_;
	
	my @listing = (); #result; 
	
	open FILE, $filename
		or die "Can't open file '$filename': $!";
	
	while (<FILE>) {
		chomp;
		if ( /^\s*#/ ) {
			if ( /^#TIMESTAMP:(\d{2})-(\d{2})-(\d{4})\s+(\d{2}):(\d{2})$/ ) {
				# Timestamp helps to determine when the cache expires.
				my $timestamp = mktime(0, $5, $4, $1, $2-1, $3-1900);
				# Not used
			}
			elsif ( /^#PWD:(.*)$/ ) {
				my $path = $1;
				# One could check the path here
			}
			else {
				print STDERR "Wrong syntax in file $filename\n";
			}
		}
		else {
			# It is a regular DIR line
			push (@listing, $_);
		}
	}
	close FILE;
	
	return \@listing;
}
#------------------------------------------------------------------------

# Returns file listing of a given dir on the ftp server.
sub list_ftp {
	my ($srv, $dir, $cache_dir) = @_;
	# $srv  - e.g. "ftp.linuxfoundation.org"
	# $dir - what directory should be listed
	
	my @ftp_files = (); # results: @ftp_files[] -> ($filename, $filesize)
	
	my $listing = ();
	
	unless ( -d $cache_dir ) {
		(system("mkdir -p '$cache_dir'") == 0 )
			or die "Failed to create the cache directory: '$cache_dir'";
	}
	
	print STDERR "> SRC $dir\n";
	
	my $ftp_list_file = "$cache_dir/ftp_list";
	unless( $list_refresh ) {
		# Read cached listing
		if ( -f $ftp_list_file ) {
			$listing = read_listing_from_file( $ftp_list_file );
		}
	} 
	if( !$listing ) { 
		# Refresh the listing.
		( $ftp ) or login_ftp( $srv );
		
		$listing = $ftp->dir($dir);
	}
	
	unless( $listing ) { 
		print STDERR "WRONG DIR\n";
		# Return empty list
		return @ftp_files;
	}
	
	# Save listing
	if ( $list_refresh || !-f $ftp_list_file ) {
		my ($sec, $min, $hour, $mday, $mon, $year) = localtime ();
		open LISTFILE, "> $ftp_list_file"
			or die "Failed to open file '$ftp_list_file' for writing: $!";
		print LISTFILE
			"#TIMESTAMP:"
			.sprintf ("%02d-%02d-%04d %02d:%02d\n", $mday, ($mon + 1), 
					($year + 1900), $hour, $min )
			."#PWD:".$dir."\n"
			.(join "\n", @$listing)
			;
		close LISTFILE;
	}
	
	unless(@$listing) { 
		print STDERR "< Empty\n";
		# Return empty list
		return @ftp_files;
	}

	# Parse ftp files listing
	foreach my $line (@$listing) {
		# -rwxr-xr-x 1 user group size date time filename.ext
		if($line =~ m/^(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)/) {
			my $attrs = $1;
			my $filesize = $2;
			my $filename = $3;
			
			# If it is an ordinary file or a symlink
			if($attrs =~ /^[-l]/ ) {
				$filesize = 0  if $attrs =~ /^l/; # Set the size undefined if this is a symlink
				my @line = ($filename, $filesize);
				push @ftp_files, \@line;
			}
		}
		else { 
			print STDERR "Wrong syntax in ftp listing: $line\n";
		}
	}
	print STDERR "< Ok\n";

	return @ftp_files;
}


#----------------------------------------------------------------------------
1; # Returned value
