# LSB Test Execution Framework
# Desktop Test Module (Desktop_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Desktop_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common

use X11_common;
#----------------------------------------------------------------------

my %opt_names = (
		"libpng" => "PNG",
		"qt3"    => "QT3",
		"fontconfig" => "FONTCONFIG",
		"gtkvts" => "GTKVTS",
		"xml"    => "XML",
		"qt4"    => "QT4",
		"freetype" => "FREETYPE",
		"xrender"  => "XRENDER",
		"xft"    => "XFT",
		"cairo"  => "CAIRO",
	);

#----------------------------------------------------------------------

sub check {
	my ($self) = @_;
	
	if ( version_filter("LSB <= 3.1", $globals->{'standard'}) ) {
	# For LSB 3.1 and older releases
		# Check Xvfb
		does_shell_know ("Xvfb")
			or return error "Can't find the 'Xvfb' tool required by "
					.$self->name." test.  Please install this tool.";
	}
	
	is_ok $self->prepare_options() or return $Error::Last;
	
	return 1;
}

sub prepare_options {
	my ($self) = @_;
	
	( defined $self->{OPTIONS} )
		or return error "Answers are undefined"; # Should not happen
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/lsb/test/desktop";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	
	my $q;
	$q = 'X11_FONT_PATH';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}->{VALUE} ) {
			my $x11_fonts_dir = X11_common::guess_font_dir();
			is_ok($x11_fonts_dir) or return error $self->name
				." test requires certain X11 fonts to be installed.", $Error::Last;
			$self->{OPTIONS}{$q}->{VALUE} = $x11_fonts_dir."/misc";
		}
	}
	
	$q = 'DESKTOP_ENVIRONMENT';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}->{VALUE} ) {
			$self->{OPTIONS}{$q}->{VALUE} = X11_common::guess_DE();
		}
	}
	
	if ( $globals->{'cert_mode'} && version_filter("LSB >= 3.2", $globals->{'standard'}) ) {
		$self->{OPTIONS}{'with_qt4'}->{VALUE} = "desktop_qt4";
	}
	
	return 1;
}

sub prepare {
	my ($self) = @_;
	
	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	# Download and install required packages
	is_ok $self->prepare_test_packages()  # According to the Manifest
		or return error "Failed to download and install packages for '"
					.$self->name."' test.", $Error::Last;
	
	is_ok $self->prepare_options() or return $Error::Last;
	
	# Too slow to put in prepare_options()
	my $q = 'XVFB_DISPLAY';
	if ( defined $self->{OPTIONS}{$q} ) {
		if ( !defined $self->{OPTIONS}{$q}->{VALUE} ) {
			$self->{OPTIONS}{$q}->{VALUE} = X11_common::guess_display();
		}
	}
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR');
	
	# Check if split mode (option '-s') is available
	if ( $self->{SPLIT_MODE} ) {
		if ( 0 != cmd(shq($testsuite_dir)."/run_tests -h | grep -e '\\-s '") ){
			return error "Split mode isn't available with this version"
					." of desktop test (".$self->{VERSION}.").";
		}
	}
	
	cmd ("killall Xvfb"); # Should be done before calling guess_display()
	sleep 1;
	
	# Remove old config
	cmd("rm -f ".shq($testsuite_dir)."/.configured");
	
	# Remove old results
	cmd("rm -rf ".shq($testsuite_dir)."/gtkvts/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/qt3/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/fontconfig/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/xft/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/xrender/results/*");
	cmd("rm -rf ".shq($testsuite_dir)."/freetype/results/*");
	cmd("rm -f ".shq($testsuite_dir)."/xml/journal.libxml2");
	cmd("rm -f ".shq($testsuite_dir)."/libpng/journal.pngtest");
	cmd("rm -f ".shq($testsuite_dir)."/Qt/journal.qt");
	cmd("rm -f ".shq($testsuite_dir)."/cairo/journal.cairo");
	
	# Prepare a list of available subtests
	my @subtests = ( "libpng", "qt3", "fontconfig", "gtkvts", "xml" );
	
	if ( version_filter("LSB >= 3.2", $globals->{'standard'}) || $self->option('with_qt4', "desktop_qt4") ) {
		push @subtests, "qt4";
	}
	if ( version_filter("LSB >= 3.2", $globals->{'standard'}) ) {
		push @subtests, "freetype", "xrender", "xft";
		
		if ( version_filter("LSB 3.2", $globals->{'standard'}) ) {
			# 'xrender' and 'xft' are trial in LSB 3.2
			$self->{EXPECTED_JOURNALS}{$_}{TRIAL_USE} = 1 for ( "xrender", "xft" );
		}
	}
	if ( version_filter("LSB >= 4.0", $globals->{'standard'}) ) {
		push @subtests, "cairo";
	}
	
	for my $subname ( @subtests ) {
		$self->{EXPECTED_JOURNALS}{$subname} ||= {};
	}
	
	$self->{SUBTESTS} = [@subtests];
	
	return 1;
}

sub run {
	my ($self, $s_opt) = @_;
	
	if ( $self->{SPLIT_MODE} ) {
		# Run each subtest
		foreach my $subname ( @{$self->{SUBTESTS}} ) {
			
			my $s_opt = $opt_names{$subname};
			if ( !$s_opt ) {
				warning "No option for desktop::$subname";
				next;
			}
			
			is_ok $self->run_sub($s_opt) or do {
				$self->report_error(complain $Error::Last);
				$self->{RUN_PROBLEMS} = 1;
			};
			
			# Cleanup
			cmd("killall Xvfb"); sleep 1;
		}
	   
		return 1;
	} else {

		return $self->run_sub();
	}
}

sub run_sub {
	my ($self, $s_opt) = @_;
	
	# Check whether the '-a' option is supported
	my $automatic_option = 
		( 0 == cmd( shq($self->{TESTSUITE_DIR}."/run_tests")." -h | grep -e '\\-a '") );
	
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Change to the test suite directory
	$subshell->Settle();
	$subshell->Send( "cd ".shq($self->{TESTSUITE_DIR})."\n" );
	
	# Prepare the command
	my $command = "./run_tests";
	if ( $s_opt ) { # execute only one test
		$command .= " -s ".$s_opt;
	} else {
	# All automatic tests
		$command .= " -a" if $automatic_option;
		
		if ( $self->option('with_qt4', "desktop_qt4") ) {
			$command .= " -m QT4";
		}
	}
	
	$subshell->Settle();
	$subshell->Send("$command; exit\n"); # Run the test
	
	my $questions_wait_time = 60*10; # In seconds
	my $questions_deadline = time() + $questions_wait_time;
	
	my $started = 0;
	
	my $line;
	DIALOG_LOOP:
	while ( $line = $subshell->Read($questions_wait_time) ) {
		
		if ( $line =~ /Starting X Virtual Frame Buffer/ ) {
			$started = 1;
			last;
		}
		
		# Answer the prompts
		
		$subshell->Settle($line) or redo DIALOG_LOOP;
		
		# Formalized questions
		foreach my $opt ( values %{$self->{OPTIONS}} ) {
			next if !defined $opt->{PRE} || !defined $opt->{VALUE};
			
			my $pre = $opt->{PRE};
			if ( $line =~ /$pre/ ) {
				
				$subshell->Send( $opt->{VALUE}."\n" );
				next DIALOG_LOOP;
			}
		}
		
	} continue {
		
		# Check the questions timeout
		if ( time() > $questions_deadline ) {
			inform "Time for questions is over.";
			is_ok $subshell->ExpectLogout() or return $Error::Last;
			return error "Timeout";
		}
	}
	if ( $subshell->{TIMEOUT} ) { inform "!!! Warning: timeout at subshell"; }
	
	# Just wait for the test finished
	$subshell->WaitForSubshell();
	
	if ( !$started ) {
		return error "The test hasn't started";
	}
	
	return 1; # Ok
}

sub after_run {
	my ($self) = @_;
	
	return 1 if !$self->{EXPECTED_JOURNALS};
	
	## Journals
	$self->{JOURNALS} = {}; # journal_filename => journal_type
	
	my $testsuite_dir = $self->{TESTSUITE_DIR};
	
	my $journal_location = {
			"gtkvts"     => $testsuite_dir."/gtkvts/results/0*/journal",
			"qt3"        => $testsuite_dir."/qt3/results/0*/journal",
			"fontconfig" => $testsuite_dir."/fontconfig/results/0*/journal",
			"xft"        => $testsuite_dir."/xft/results/0*/journal",
			"xrender"    => $testsuite_dir."/xrender/results/0*/journal",
			"freetype"   => $testsuite_dir."/freetype/results/0*/journal",
			"xml"        => $testsuite_dir."/xml/journal.libxml2",
			"libpng"     => $testsuite_dir."/libpng/journal.pngtest",
			"qt4"        => $testsuite_dir."/Qt/journal.qt",
			"cairo"      => $testsuite_dir."/cairo/journal.cairo",
		};
	
	foreach my $subname ( @{$self->{SUBTESTS}} ) {
		my $obj = $self->{EXPECTED_JOURNALS}{$subname};
		if ( $obj ) {
			my $journal_file = "desktop_".$subname.".journal";
			is_ok $self->copy_file_globbed( $journal_location->{$subname}, $globals->{'test_result_dir'}."/".$journal_file )
				or $self->report_error(complain $Error::Last);
			$obj->{FILE} = $globals->{'test_result_dir'}."/".$journal_file;
			$obj->{KIND} = 'TET';
		}
		else {
			$self->report_error(complain "No journals from desktop/$subname test");
		}
	}
	
	# Pack $self->{JOURNALS}
	# The JOURNALS should be string to be saved to the .info file.
	my $packed = "";
	foreach my $subname ( @{$self->{SUBTESTS}} ) {
		my $obj = $self->{EXPECTED_JOURNALS}{$subname};
		if ( $obj ) {
			$packed .= $subname.":";
			$packed .= join ",", map {$_."=".$obj->{$_}} keys %$obj;
			$packed .= ";";
		}
	}
	$self->{JOURNALS} = $packed;
	
	return 1;
}

sub make_report {
	my ($self) = @_;
	
	( $self->{JOURNALS} ) or return error("No journals");
	
	# Unpack $self->{JOURNALS}: "key=value,key=value,..."
	my $journals = {};
	my @subtests = ();
	for my $s ( split /;/, $self->{JOURNALS} ) {
		my ($subname, $data) = split /:/, $s, 2;
		foreach my $pair ( split /,/, $data ) {
			my ($key, $value) = split /=/, $pair;
			$journals->{$subname}{$key} = $value;
		}
		push @subtests, $subname;
	}
	
	# Make journals table
	my $journals_toc = {};
	my @verdicts = ();
	my $report = "";
	
	foreach my $subname ( @subtests ) {
		my $obj = $journals->{$subname};
		
		# Add an anchor for journals TOC
		$report .= '<a name="'.html($subname).'"></a>'."\n";
		# Title
		my $title = "desktop/".$subname;
		$report .= '<h2>'.html($title).'</h2>'."\n";
		
		my $jstruct = { %$obj };
		if ( !$obj->{FILE} ) {
			local $Error::Debug = 0;
			$report .= Report::ErrMessage(complain "$subname: Journal file undefined");
			next;
		}
		$jstruct->{TESTSUITE} = $title;
		
		# Print a link to the journal
		$report .= '<p>Open the test '.Report::local_href( $jstruct->{FILE}, 'journal' ).'.</p>';
		
		is_ok Report::ParseJournal( $jstruct )
			or do {
				local $Error::Debug = 0;
				$report .= Report::ErrMessage( complain "Failed to parse journal: ".$jstruct->{FILE}, $Error::Last );
				push @verdicts, 'run_problems';
				next;
			};
		
		if ( $subname eq 'cairo' ) {
			# cairo journal is malformed and it cause many INFO_TABLES to appear. Truncate them.
			$#{$jstruct->{INFO_TABLES}} = 1 if $jstruct->{INFO_TABLES} && $#{$jstruct->{INFO_TABLES}} > 2;
		}

		# Print the errors table
		$report .= Report::JournalReport($jstruct);
		# Print the statistics
		$report .= Report::print_stats($jstruct->{STATS});
		
		my $journal_verdict = Report::verdict( $jstruct->{STATS} );
		push @verdicts, $journal_verdict;
		push @{$journals_toc->{ROWS}}, {
				LEFT => '<a href="#'.html($subname).'">'.html($title).'</a>',
				RIGHT => Report::verdict_text($journal_verdict),
				BG_COLOR => Report::verdict_color($journal_verdict),
			};
	}
	
	$self->{VERDICT} = Report::overall_verdict(@verdicts);
	
	$journals_toc->{TITLE} = 'Test journals';
	$self->report( Report::print_info_table($journals_toc) );
	
	$self->report($report);
	
	return 1;
}

sub Cleanup {
	my ($self) = @_;
	
	cmd("killall Xvfb");
}

1; # return value
