# LSB Test Execution Framework
# Manual Common FVT Module (Manual_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Manual_test;
use strict;

use Misc;
use SimpleJournal;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

sub new {
	my $self  = Test_common::new(@_);
	$self->{MANUAL} = "1";

	return $self;
}

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	$self->{JOURNAL_FILE} = $self->name.".simplejournal";
	$self->{JOURNAL_TYPE} = 'SIMPLE';
	
	# 1. Download the application package that corresponds to the LSB
	#    version and processor architecture for which certification is being 
	#    sought from the Download Page. 
	# 2. Install package on system.
	# NOTE: Application MUST be installed without use of --nodeps option,
	#  the dependency checks are part of the test.
	
	# Download and install required packages
	my $inst_result = $self->prepare_test_packages();  # According to Manifest
	if ( !is_ok($inst_result) ) {
		my $err = error "Failed to download and install packages", $Error::Last;
		
		# Test case failed. Create a journal with one record.
		
		my $simpjournal = new SimpleJournal($self->journal_file);
		is_ok $simpjournal or return error "Failed to open journal.", $Error::Last;
		
		$simpjournal->test_name( $self->name );
		
		$simpjournal->test_case_start( 'Installation' );
		
		$simpjournal->test_description(<<HEREDOC);
1. Download the application package that corresponds to the LSB
   version and processor architecture for which certification is being 
   sought from the Download Page. 
2. Install package on system.
 NOTE: Application MUST be installed without use of --nodeps option,
  the dependency checks are part of the test.
HEREDOC
		
		$simpjournal->test_comment( $err->tostring() );
		
		$simpjournal->test_case_result('failed');
		
		$simpjournal->close();
	}
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	# This is a manual test
	inform "The test has been prepared. Use the WebUI to accomplish it.";
	
	return 1; # Ok
}

sub finish {
	my ($self) = @_;
	
	# Do nothing.
	# This journal is expected to be generated by the webui.
}

sub make_report {
	my ($self) = @_;
	
	( $self->{JOURNAL_FILE} ) or return error("No journal name specified");
	
	if ( !-f $self->journal_file ) {
		$self->{VERDICT} = 'incomplete';
		$self->report( '<p>The test has been prepared. You can '
					.'<a href="tests_appbat.pl?test_run='.$globals->{'testrun_id'}
						.'&test='.$self->name.'"><b>run it now</b></a>.</p>' );
	} else {
		is_ok Test_common::make_report($self);
	}
}

1; # return value
