# LSB Test Execution Framework
# Olver Test Module (Olver_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Olver_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

my $checkpoints = undef;

#----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	# Download and install required packages
	is_ok $self->prepare_test_packages()  # According to the Manifest
		or return error "Failed to download and install packages for '"
					.$self->name."' test.", $Error::Last;
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/lsb/test/olver-core";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	$self->{RESULTS_DIR} = $self->option('RESULTS_DIR') || "/var/opt/lsb/test/olver-core";
	
	# Remove old results
	cmd("rm -rf ".shq($self->{RESULTS_DIR})."/*");
	
	init_checkpoint_data($self->{VERSION});
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Change to the test suite directory
	$subshell->Settle();
	$subshell->Send( "cd ".shq($self->{TESTSUITE_DIR})."\n" );
	
	# Run the test
	$subshell->Settle();
	$subshell->Send("./bin/olver_run_tests; exit\n");
	
	# Just wait for the test finished
	while ( my $line = $subshell->Read(2) ) {
		$self->check_progress($line);
	}
	
	# Just confirm that the subshell has finished
	$subshell->WaitForSubshell();
	
	return 1; # Ok
}

sub check_progress {
	my ($self, $line) = @_;
	
	defined $line or $line = "";
	
	if ( $line =~ /^Running scenario (\S+).*[\r\n]/ ) {
		my $testcase = $1;
		$self->progress_profiled($checkpoints, $testcase);
		return;
	}
	# else:
	$self->progress_profiled();
}

sub after_run {
	my ($self) = @_;
	
	# Journal
	$self->{JOURNAL_FILE} = $self->name.".journal";
	$self->{JOURNAL_TYPE} = 'TET';
	
	# Find the journal file and copy it to where it should be.
	is_ok $self->copy_file_globbed( $self->{RESULTS_DIR}."/*/tet/tetreport.log", $self->journal_file )
		or $self->report_error(complain $Error::Last);
	
	is_ok $self->copy_file_globbed( "/var/opt/lsb/test/olver-core/*", $globals->{"test_result_dir"}."/olver_full" )
		or $self->report_error(complain $Error::Last);
	
	return 1;
}

sub init_checkpoint_data {
	my ($ver) = @_;
	
	# olver v. 4.0.0-5 for LSB 4.0
	$checkpoints = [
		[ 0, 'dir_scenario' ],
		[ 10, 'fstream_action_scenario' ],
		[ 16, 'line_scenario' ],
		[ 22, 'window_attr_scenario' ],
		[ 29, 'execle_scenario' ],
		[ 40, 'ualarm_scenario' ],
		[ 47, 'vswscanf_scenario' ],
		[ 54, 'scanf_scenario' ],
		[ 62, 'vsprintf_scenario' ],
		[ 68, 'vsscanf_scenario' ],
		[ 75, 'getopt_scenario' ],
	];
}

1; # return value
