#!/usr/bin/perl -w

# LSB Test Execution Framework
# Manual Application Battery Tests Page Module (tests_appbat.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use UserProfile;
use Error;
use SimpleJournal;

my @tests_list = ();            # List of test suites defined for the current standard version
my %tests_length = ();          # Number of questions in each of the tests available
my %tests_result = ();          # Results of the tests execution stored in the journal files (if present)
my %result_display = (-1 => '<font color="blue">Not finished</font>', 0 => '<font color="red">Failed</font>', 1 => '<font color="green">Passed</font>');

my @test_cases = ();            # Questions in the test chosen
my @test_cases_answers = ();    # Results stored in the journal for the test chosen
my $test_comments = '';         # Comments for the test chosen
my $manifest;                   # Manifest data

my $std_version = DEFAULT_STANDARD_VER;
my $architecture = (detect_architecture() or 'x86');

my $idx;
my $error = '';

# Check if the necessary arguments are specified.
if (!$_GET{'test_run'}) {
	$error = 'Test run is not specified!';
}
elsif ($_POST{'finish_test'} and !$_GET{'test'}) {
	$error = 'Test name is not specified when saving!';
}

if (!$error) {
	# Read the standard version and architecture
	if (open(FILE, "$CONFIG{'RESULTS_DIR'}/$_GET{'test_run'}/runconfig")) {
		while (my $line = <FILE>) {
			if ($line =~ m/STANDARD: (.*)/i) {
				$std_version = $1;
			}
			if ($line =~ m/HOST_architecture: (.*)/i) {
				$architecture = $1;
			}
		}
		close(FILE);
	}
	# Read list of the tests available.
	$manifest = load_manifest();
	if (is_ok($manifest)) {
		my $composition_tests = $manifest->{'COMPOSITION'}->{$std_version};
		@tests_list = sort grep { $composition_tests->{$_}->{$architecture} } keys %{$manifest->{'MANUAL'}};
		if (scalar(@tests_list) == 0) {
			$error = 'No manual tests were found for '.$std_version.'!';
		}
	}
	else {
		$error = error('Failed to load manifest!', $Error::Last)->tostring;
	}
}

if (!$error) {
	# Filter only tests selected in the current test run.
	my $profile = read_profile($CONFIG{'RESULTS_DIR'}.'/'.$_GET{'test_run'}.'/profile');
	if ($profile) {
		if (!$profile->{'GENERAL'}->{'CERTIFY'}) {
			@tests_list = grep { $profile->{'TEST_SUITES'}->{$_}->{'RUN'} } @tests_list;
		}
	}
	if (scalar(@tests_list) > 0) {
		# If no test specified, we will open the first one.
		if (!$_GET{'test'}) {
			$_GET{'test'} = $tests_list[0];
		}
	}
	else {
		$error = 'There are no manual tests to run!';
	}
}

if (!$error) {
	# Get the list of manual steps according to the standard version.
	my @ts_questions = grep { version_filter($_->{'STANDARD'}, $std_version) } @{$manifest->{'MANUAL'}->{$_GET{'test'}}};
	$idx = 0;
	if (scalar(@ts_questions) > 0) {
		foreach (@ts_questions) {
			$test_cases[++$idx] = $_->{'STEP'};
		}
	}
	else {
		$error = 'No manual steps were found for the selected test!';
	}
}

# If POST-query 'Finish test', save the journal and reload page.
if (!$error and $_POST{'finish_test'}) {
	# Get the next test to open.
	my $i;
	for ($i=0; $i<=$#tests_list; ++$i) {
		last if ($tests_list[$i] eq $_GET{'test'});
	}
	++$i if ($i < $#tests_list);
	my $next_test = $tests_list[$i];

	# Create a journal file using SimpleJournal module
	my $journal = SimpleJournal->new("$SERVER_PARAM{'APP_DATA'}/results/$_GET{'test_run'}/results/$_GET{'test'}.simplejournal");
	if (!is_ok($journal)) {
		$error = error('Failed to open journal.', $Error::Last);
	}
	else {
		$journal->test_name($_GET{'test'});

		$idx = 1;
		while ($_POST{'test'.$idx}) {
			$journal->test_case_start("Q$idx");
			$journal->test_description($test_cases[$idx]);
			$journal->test_case_result(($_POST{'test'.$idx} eq 'yes') ? 'passed' : 'failed');
			++$idx;
		}
		$journal->test_case_start('Summary');
		$journal->test_comment($_POST{'deviations'});
		$journal->test_case_result(($_POST{'problems'} eq 'yes') ? 'failed' : 'passed');
		$journal->close();

		print 'HTTP/1.0 302 Moved' . CRLF . "Location: tests_appbat.pl?test_run=$_GET{'test_run'}&test=$next_test" . CRLF . CRLF;
		exit;
	}
}

# Print header.
print 'HTTP/1.0 200 OK' . CRLF;
print 'Content-type: text/html' . CRLF . CRLF;

print_header('Manual Application Battery Tests', 'manappbat');
print "<h1>Manual Application Battery Tests</h1>\n";

if (!$error) {
	my $allow_finish = 1;
	foreach my $test (@tests_list) {
		# Count total number of questions in the test.
		$tests_length{$test} = scalar(grep { version_filter($_->{'STANDARD'}, $std_version) } @{$manifest->{'MANUAL'}->{$test}});
		# All tests are supposed at first not finished.
		$tests_result{$test} = -1;

		# Read journal file if the test already performed and store the result.
		if (open(TEST_JOURNAL, "$SERVER_PARAM{'APP_DATA'}/results/$_GET{'test_run'}/results/$test.simplejournal")) {
			my %numbers = ();
			$tests_result{$test} = 1;                   # First, suppose all is OK
			while(<TEST_JOURNAL>) {
				if (m/^TEST:\s*(.*)$/i) {
					if ($1 ne $test) {
						$tests_result{$test} = -1;      # Wrong journal, mark as not finished
						last;
					}
				}
				elsif (m/^TESTCASE:\s*Q(\d+)$/i) {
					$idx = $1;
				}
				elsif (m/^TESTCASE:\s*Summary$/i) {
					$idx = 0;
				}
				elsif (m/^ COMMENT: (.*)$/i) {
					if (($test eq $_GET{'test'}) && ($idx == 0)) {
						$test_comments .= (($test_comments eq '') ? '' : "\n").$1;
					}
				}
				elsif (m/^RESULT:\s*(passed|failed)$/i) {
					$numbers{$idx} = 1;
					if ($1 eq 'failed') {
						$tests_result{$test} = 0;       # Not all is OK
					}
					$test_cases_answers[$idx] = ((($1 eq 'passed') xor ($idx == 0)) ? 'yes' : 'no') if ($test eq $_GET{'test'});
				}
			}
			close(TEST_JOURNAL);
			if ($tests_result{$test} != -1) {
				foreach (0..$tests_length{$test}) {
					if (!$numbers{$_}) {
						$tests_result{$test} = -1;      # Not all questions are answered
						last;
					}
				}
			}
		}
		$allow_finish = 0 if ($tests_result{$test} == -1);
	}

	# Output the left pane (list of the tests).
	print <<DATA;
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td style="width: 21em; vertical-align: top;">
      Please, perform the following manual tests to complete the test run:<br /><br />
      <table class="main" style="width: 21em;">
DATA
	my $finish_button = ($allow_finish ?
		'<input type="button" name="finish_appbat" value="Finish" style="width: 6em;" onclick="javascript:location=\'tests_results.pl?generate='.$_GET{'test_run'}.'\';" />' :
		'<input type="button" name="finish_appbat" value="Finish" style="width: 6em;" disabled="disabled" /><br />You need to finish all the tests to continue.');
	foreach my $test_name (@tests_list) {
		my $dispname = $test_name;
		my ($descr_obj) = grep { version_filter($_->{'STANDARD'}, $std_version) } @{$manifest->{'DESCRIPTIONS'}->{$test_name}};
		if ($descr_obj) {
			$dispname = $descr_obj->{'DISPLAYNAME'};
		}
		print "        <tr>\n          <td style=\"width: 6.3em;\"><a href=\"tests_appbat.pl?test_run=$_GET{'test_run'}&amp;test=$test_name\">".(($test_name eq $_GET{'test'}) ? "<b>$dispname</b>" : $dispname)."</a></td>\n          <td style=\"width: 12em;\">Status: <b>".$result_display{$tests_result{$test_name}}."</b></td>\n        </tr>\n";
	}
	print <<DATA;
      </table>
      <br /><br />
      $finish_button
    </td>
    <td><img src="images/environment-spacer.gif" width="30" height="1" border="0" alt="" /></td>
    <td width="100%" style="border: solid 1px gray; padding: 10px;" valign="top">
DATA

	# Output the right pane (test chosen).
	print "      <form method=\"post\" action=\"tests_appbat.pl?test_run=$_GET{'test_run'}&amp;test=$_GET{'test'}\">\n";
	my $dispname = $_GET{'test'};
	my ($descr_obj) = grep { version_filter($_->{'STANDARD'}, $std_version) } @{$manifest->{'DESCRIPTIONS'}->{$_GET{'test'}}};
	if ($descr_obj) {
		$dispname = $descr_obj->{'DISPLAYNAME'};
	}

	print <<DATA;
<h2>Functional Verification Test: $dispname</h2>

<table>
<thead>
<tr>
<th>&nbsp;</th>
<th>Expected&nbsp;result&nbsp;&nbsp;<br />achieved?</th>
<th>Test case scenario steps</th>
</tr>
</thead>
<tbody>
DATA
	my $sel_yes;
	my $sel_no;
	foreach (1..$#test_cases) {
		$sel_yes = (($test_cases_answers[$_] and ($test_cases_answers[$_] eq 'yes')) ? ' checked="checked"' : '');
		$sel_no = (($test_cases_answers[$_] and ($test_cases_answers[$_] eq 'no')) ? ' checked="checked"' : '');
		print <<DATA;
<tr>
<td valign="top">$_.</td>
<td valign="top">
<span id="test$_\_bg" style="white-space: nowrap;"><label><input type="radio" name="test$_" id="test$_\_yes" value="yes" onclick="javascript:clear_bg('test$_');"$sel_yes />Yes</label> <label><input type="radio" name="test$_" id="test$_\_no" value="no" onclick="javascript:clear_bg('test$_');"$sel_no />No</label></span>
</td>
<td>
$test_cases[$_]
</td>
</tr>
DATA
	}
	print <<DATA;
</tbody>
</table>

DATA
	$sel_yes = (($test_cases_answers[0] and ($test_cases_answers[0] eq 'yes')) ? ' checked="checked"' : '');
	$sel_no = (($test_cases_answers[0] and ($test_cases_answers[0] eq 'no')) ? ' checked="checked"' : '');
	print <<DATA;
<h2>Results</h2>

<table>
<tr><td valign="top">1.</td>

<td valign="top">
<span id="problems_bg" style="white-space: nowrap;"><label><input type="radio" name="problems" id="problems_yes" value="yes" onclick="javascript:clear_bg('problems');"$sel_yes />Yes</label> <label><input type="radio" name="problems" id="problems_no" value="no" onclick="javascript:clear_bg('problems');"$sel_no />No</label></span>
</td>

<td>Were there any deviations in the install, setup, or test described above?
<br />If yes, please describe.
<br /><textarea name="deviations" rows="3" cols="60">$test_comments</textarea></td>
</tr>
</table>

        <br />
        <input type="submit" name="finish_test" id="finish_test" value="Finish test" style="width: 6em;" onclick="javascript:return check_finish();" />&nbsp;&nbsp;&nbsp;<input type="reset" name="reset_test" value="Reset test" style="width: 6em;" onclick="javascript:return check_reset();" />
      </form>
    </td>
  </tr>
</table>
DATA

	my $num_elems = $#test_cases;

	print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
function check_finish() {
	var i;
	var elem_yes;
	var elem_no;
	var elem;
	var unchecked = 0;
	for (i=1; i<=$num_elems; ++i) {
		elem_yes = document.getElementById('test' + i + '_yes');
		elem_no = document.getElementById('test' + i + '_no');
		if (elem_yes && elem_no) {
			if (!elem_yes.checked && !elem_no.checked) {
				++unchecked;
				elem = document.getElementById('test' + i + '_bg');
				if (elem) {
					elem.style.backgroundColor = '#ff7f7f';
				}
			}
		}
		else {
			++unchecked;
		}
	}
	elem_yes = document.getElementById('problems_yes');
	elem_no = document.getElementById('problems_no');
	if (elem_yes && elem_no) {
		if (!elem_yes.checked && !elem_no.checked) {
			++unchecked;
			elem = document.getElementById('problems_bg');
			if (elem) {
				elem.style.backgroundColor = '#ff7f7f';
			}
		}
	}
	else {
		++unchecked;
	}
	if (unchecked > 0) {
		var single_plural = ((unchecked == 1) ? 'is 1 question' : 'are ' + unchecked + ' questions');
		alert('Please, answer all the questions of the test!\\nThere ' + single_plural + ' remained unanswered.');
		return false;
	}
	else {
		return true;
	}
}

function clear_bg(name) {
	elem = document.getElementById(name + '_bg');
	if (elem) {
		elem.style.backgroundColor = '';
	}
}

function check_reset() {
	if (confirm('Are you sure you wish to clear your answers?')) {
		var i;
		for (i=1; i<=$num_elems; ++i) {
			clear_bg('test' + i);
		}
		clear_bg('problems');
		return true;
	}
	else {
		return false;
	}
}
// ]]>
</script>
DATA
}
else {
	print "<h3><font color=\"red\" size=\"+1\">Error!</font></h3>\n$error\n";
}

print_footer();
